using System;
using System.Text;
using System.Collections.Generic;
#if !NUNIT
using Microsoft.VisualStudio.TestTools.UnitTesting;
#else
using NUnit.Framework;
using TestClass = NUnit.Framework.TestFixtureAttribute;
using TestInitialize = NUnit.Framework.SetUpAttribute;
using TestCleanup = NUnit.Framework.TearDownAttribute;
using TestMethod = NUnit.Framework.TestAttribute;
#endif
using ConfigurationCoreTests.Configuration;
using System.Management; 
using System.Diagnostics;
using Microsoft.Practices.EnterpriseLibrary.Common.Instrumentation;
using Microsoft.Practices.EnterpriseLibrary.Common.Configuration;
using System.Xml; 

namespace ConfigurationCoreTests
{
    /// <summary>
    /// Summary description for NamedTypeTestFixture
    /// </summary>
    [TestClass]
    public class NamedObjectTestFixture
    {
        private bool logged = false;
        private string loggedMessage = "";
        private bool itemConfigNotificationFlag = false;
        int count = 0;


        public NamedObjectTestFixture()
        {
            //
            // TODO: Add constructor logic here
            //
        }

        #region Additional test attributes
        //
        // You can use the following additional attributes as you write your tests:
        //
        // Use ClassInitialize to run code before running the first test in the class
        // [ClassInitialize()]
        // public static void MyClassInitialize(TestContext testContext) { }
        //
        // Use ClassCleanup to run code after all tests in a class have run
        // [ClassCleanup()]
        // public static void MyClassCleanup() { }
        //
        // Use TestInitialize to run code before running each test 
        // [TestInitialize()]
        // public void MyTestInitialize() { }
        //
        // Use TestCleanup to run code after each test has run
        // [TestCleanup()]
        // public void MyTestCleanup() { }
        //
        #endregion

        #region NameTypeFactoryBase tests
        [TestMethod]
        public void CreateNamedTypeObject()
        {
            ItemProviderFactory factory = new ItemProviderFactory();
            IItem item = factory.Create("Item1");
            Assert.IsTrue(item.CalculateTotal() == 0);   
        }

        [TestMethod]
        public void CreateDefaultObject()
        {
            ItemProviderFactory factory = new ItemProviderFactory();
            IItem item = factory.CreateDefault(); 
            Assert.IsTrue(item.CalculateTotal() == 0);
        }

        [TestMethod]
        public void CreateNamedTypeObjectExplicit()
        {
            ItemProviderFactory factory = new ItemProviderFactory();
            CommonItem item = (CommonItem)factory.Create("Item1");
            Assert.IsTrue(item.CalculateTotal() == 0);
        }

        [TestMethod]
        public void CreateNamedDerivedTypeObject()
        {
            ItemProviderFactory factory = new ItemProviderFactory();
            IItem item = factory.Create("Item2");
            Assert.IsTrue(item.CalculateTotal() == 1000);
        }

        [TestMethod]
        public void CreateNamedDerivedTypeObjectExplicit()
        {
            ItemProviderFactory factory = new ItemProviderFactory();
            SoftwareItem item = (SoftwareItem)factory.Create("Item2");
            Assert.IsTrue(item.CalculateTotal() == 1000);
        }

        [TestMethod]
        public void CreateCommonObjectDirect()
        {
            CommonItem item = new CommonItem(10, 100);
            Assert.IsTrue(item.CalculateTotal() == 1000);
        }

        [TestMethod]
        public void CreateDerivedObjectDirect()
        {
            SoftwareItem item = new SoftwareItem(10, 100,"2.0.0.0");
            Assert.IsTrue(item.CalculateTotal() == 900);
        }
#endregion

        #region Instrumentation Tests
        [TestMethod] 
        public void VerifyItemCalculatedPerfCounter()
        {
            string counterName = "Item Categories Calculated/sec";
            int initialCount = GetCounterValue(counterName, "Total");
            ItemProviderFactory factory = new ItemProviderFactory();
            SoftwareItem item = (SoftwareItem)factory.Create("Item2");
            item.CalculateTotal(); 
            int loggedCount = GetCounterValue(counterName, "Total");
            Assert.IsTrue((loggedCount - initialCount) == 1);

        }

        [TestMethod]
        public void VerifyErrorLoggedToEventLog()
        {
            using (EventLog log = new EventLog("Application"))
            {
                int initialCount = log.Entries.Count;
                ItemProviderFactory factory = new ItemProviderFactory();
                IItem item = factory.Create("Item1");
                item.CalculateTotal(); 
                int finalCount = log.Entries.Count;
                Assert.IsTrue(finalCount - initialCount == 1);
                Assert.IsTrue(log.Entries[finalCount - 1].Message.Contains("The quantity value should be greater than zero"));
                Assert.IsTrue(log.Entries[finalCount - 1].Source.Equals("Enterprise Library Test"));
            }

        }

        [TestMethod]
        public void VerifyErrorLoggedToWMI()
        {
            ItemProviderFactory factory = new ItemProviderFactory();
            IItem item = factory.Create("Item1");
            ManagementScope scope = new ManagementScope(@"\\.\root\EnterpriseLibrary");
            scope.Options.EnablePrivileges = true;
            EventQuery eq = new EventQuery("SELECT * FROM ItemCalculatedEvent");
            ManagementEventWatcher watcher = new ManagementEventWatcher(scope, eq);
            watcher.EventArrived += new EventArrivedEventHandler(WatcherEventArrived);
            watcher.Start();
            logged = false;
            loggedMessage = "";
            item.CalculateTotal();
            System.Threading.Thread.Sleep(1000);
            watcher.Stop();
            watcher.Dispose();
            Assert.IsTrue(logged);
            Assert.IsTrue(loggedMessage.Contains("The quantity value should be greater than zero"));     
        }

        private void WatcherEventArrived(object sender, EventArrivedEventArgs args)
        {
            loggedMessage = args.NewEvent.GetText(TextFormat.Mof);
            logged = true;
        }

        public static int GetCounterValue(string counterName, string category)
        {
            string categoryName = "Test Item Counters";
            string instanceName = new AppDomainNameFormatter().CreateName(category);
            if (PerformanceCounterCategory.InstanceExists(instanceName, categoryName))
            {
                using (PerformanceCounter counter = new PerformanceCounter())
                {
                    counter.CategoryName = categoryName;
                    counter.CounterName = counterName;
                    counter.InstanceName = instanceName;
                    return (int)counter.RawValue;
                }
            }
            return 0;
        }
#endregion

        #region file configuration watcher tests
        [TestMethod]
        public void VerifyNotificationHasOccured()
        {
            itemConfigNotificationFlag = false;
            SystemConfigurationSource source = new SystemConfigurationSource();
            ItemSettings setting = new ItemSettings();
            setting.Items.Add(new ItemData("Item1", Type.GetType("ConfigurationCoreTests.CommonItem,ConfigurationCoreTests"), 100, 0));
            setting.Items.Add(new SoftwareItemData("Item2", Type.GetType("ConfigurationCoreTests.SoftwareItem,ConfigurationCoreTests"), 100, 10, "1.0.0.0"));
            setting.DefaultItem = "Item1";
            source.Save("ConfigurationCoreTests.dll.config", "itemConfiguration", setting);
            source.AddSectionChangeHandler("itemConfiguration", new ConfigurationChangedEventHandler(OnConfigurationChanged));
            source.GetSection("itemConfiguration");
            ItemSettings setting1 = new ItemSettings();
            setting1.Items.Add(new ItemData("Item1", Type.GetType("ConfigurationCoreTests.CommonItem,ConfigurationCoreTests"), 100, 0));
            setting1.Items.Add(new SoftwareItemData("Item2", Type.GetType("ConfigurationCoreTests.SoftwareItem,ConfigurationCoreTests"), 10, 10, "1.0.0.0"));
            setting1.DefaultItem = "Item1";
            source.Save("ConfigurationCoreTests.dll.config", "itemConfiguration", setting1);
            System.Threading.Thread.Sleep(20000);
            Assert.IsTrue(itemConfigNotificationFlag);
        }

        [TestMethod]
        public void VerifyNotificationOccuredOnlyOnce()
        {
            itemConfigNotificationFlag = false;
            SystemConfigurationSource source = new SystemConfigurationSource();
            ItemSettings setting = new ItemSettings();
            setting.Items.Add(new ItemData("Item1", Type.GetType("ConfigurationCoreTests.CommonItem,ConfigurationCoreTests"), 100, 0));
            setting.Items.Add(new SoftwareItemData("Item2", Type.GetType("ConfigurationCoreTests.SoftwareItem,ConfigurationCoreTests"), 100, 10, "1.0.0.0"));
            setting.DefaultItem = "Item1";
            source.Save("ConfigurationCoreTests.dll.config", "itemConfiguration", setting);
            source.AddSectionChangeHandler("itemConfiguration", new ConfigurationChangedEventHandler(OnConfigurationChanged));
            source.GetSection("itemConfiguration");
            ItemSettings setting1 = new ItemSettings();
            setting1.Items.Add(new ItemData("Item1", Type.GetType("ConfigurationCoreTests.CommonItem,ConfigurationCoreTests"), 100, 0));
            setting1.Items.Add(new SoftwareItemData("Item2", Type.GetType("ConfigurationCoreTests.SoftwareItem,ConfigurationCoreTests"), 10, 10, "1.0.0.0"));
            setting1.DefaultItem = "Item1";
            source.Save("ConfigurationCoreTests.dll.config", "itemConfiguration", setting1);
            System.Threading.Thread.Sleep(20000);
            Assert.IsTrue(itemConfigNotificationFlag);
            Assert.IsTrue(count == 1);
        }

        [TestMethod]
        public void VerifyNotificationCancelledWhenUnregistered()
        {
            itemConfigNotificationFlag = false;
            ConfigurationChangedEventHandler hanlder = new ConfigurationChangedEventHandler(OnConfigurationChanged);
            SystemConfigurationSource source = new SystemConfigurationSource();
            ItemSettings setting = new ItemSettings();
            setting.Items.Add(new ItemData("Item1", Type.GetType("ConfigurationCoreTests.CommonItem,ConfigurationCoreTests"), 100, 0));
            setting.Items.Add(new SoftwareItemData("Item2", Type.GetType("ConfigurationCoreTests.SoftwareItem,ConfigurationCoreTests"), 100, 10, "1.0.0.0"));
            setting.DefaultItem = "Item1";
            source.Save("ConfigurationCoreTests.dll.config", "itemConfiguration", setting);
            source.AddSectionChangeHandler("itemConfiguration", hanlder);
            source.GetSection("itemConfiguration");
            source.RemoveSectionChangeHandler("itemConfiguration", hanlder);
            ItemSettings setting1 = new ItemSettings();
            setting1.Items.Add(new ItemData("Item1", Type.GetType("ConfigurationCoreTests.CommonItem,ConfigurationCoreTests"), 100, 0));
            setting1.Items.Add(new SoftwareItemData("Item2", Type.GetType("ConfigurationCoreTests.SoftwareItem,ConfigurationCoreTests"), 10, 10, "1.0.0.0"));
            setting1.DefaultItem = "Item1";
            source.Save("ConfigurationCoreTests.dll.config", "itemConfiguration", setting1);
            System.Threading.Thread.Sleep(20000);
            Assert.IsFalse(itemConfigNotificationFlag);
            Assert.IsTrue(count == 0);
        }

        private void OnConfigurationChanged(object sender, ConfigurationChangedEventArgs e)
        {
           if (e.SectionName == "itemConfiguration")
            {
                itemConfigNotificationFlag = true;
            }
            count++;
        }

        #endregion

    }
}
